#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

void
pipeline(void (*pre)(FILE *), char **argv, void (*post)(FILE *))
{
	int fd[2];
	FILE *to_proc, *from_pre, *to_post, *from_proc;

	/* Set up two pipes: pre | proc | post. */

	if (pipe(fd) < 0) {
		fprintf(stderr, "[pipeline] can't pipe()\n");
		exit(-1);
	}

	from_proc = fdopen(fd[0], "r");
	to_post = fdopen(fd[1], "w");

	if (fork() == 0) {

		/* Close the end of the pipe we don't want. */

		fclose(from_proc);

		if (pipe(fd) < 0) {
			fprintf(stderr, "[pipeline] can't pipe()\n");
			exit(-1);
		}
		from_pre = fdopen(fd[0], "r");
		to_proc = fdopen(fd[1], "w");

		if (fork() != 0) {

			/* Close the end of the pipe we don't want. */

			fclose(from_pre);

			/* Pre: write data into pipeline. */

			(*pre)(to_proc);
			fclose(to_proc);
			wait(NULL);
			exit(0);
		} else {

			/* Close the end of the pipe we don't want. */

			fclose(to_proc);

			/* Assign proc's stdin. */
		
			close(fileno(stdin));
			dup(fileno(from_pre));

			/* Assign proc's stdout. */

			close(fileno(stdout));
			dup(fileno(to_post));

			/* proc: */

			execvp(argv[0], argv);
			fprintf(stderr, "[pipeline] can't exec() proc\n");
			exit(-1);
		}
	} 

	/* Close the end of the pipe we don't want. */

	fclose(to_post);

	/* post: read the output of the pipeline. */

	(*post)(from_proc);
	wait(NULL);
}
