/* Standard utilities. */

#ifndef _UTIL_H
#define _UTIL_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#ifndef EPS
#define EPS 1e-7
#endif

#ifdef  __cplusplus
extern "C" {
#endif

/* A function and some macros for allocating memory. [mem.c] */

extern void *new_(size_t s);

#define new(type) (type *)new_(sizeof(type))
#define new_string(size) (char *)new_((size) + 1)
#define new_array(type, size) (type *)new_((size) * sizeof(type))

/* General memory allocation. [mem.c] */

extern char *copy_string(char *s);
extern float **new_matrix(int n, int m);
extern void free_matrix(float **);

/* Error messages. [err.c] */

extern char *Progname;
extern void fatalerr(char *msg, ...);
extern void msg(char *msg, ...);

/* True if machine is MSB first. (pdp_endian not supported) [endian.c] */

extern long Big_endian;
extern int big_endian();

/* Files. [file.c] */

extern FILE *fileopen(char *name, char *mode);
extern int fileexists(char *name);
#define checkfile(name) fileexists(name)

extern char *fgetline(char *buf, int maxlen, FILE *infile);

/* Compressed file I/O. [zopen.c] */

extern FILE *zopen(char *name, char *mode);
extern void zclose(FILE *file);

/* Random numbers. [rnd.c + generator] */

extern long Frand_seed;
extern void sfrand(void);
extern float frand(void);
extern int rnd(int range);
extern void shuffle(int *array, int n);

/* Useful routines for the manipulation of 3-vectors. [xyz.c] */

#ifndef XYZ_C
#define INLINE extern __inline__
#else
#define INLINE extern
#endif

INLINE float *vnew(float x, float y, float z);
INLINE void vset(float *v, float x, float y, float z);
INLINE void vcopy(float *vsrc, float *vdst);
INLINE void vprint(float *v);
INLINE void vfprint(FILE *, float *v);
INLINE void vzero(float *v);
INLINE void vnormalize(float *v);
INLINE float vlength(float *v);
INLINE void vscale(float *v, float f);
INLINE void vmult(float *src1, float *src2, float *dst);
INLINE void vadd(float *src1, float *src2, float *dst);
INLINE void vsub(float *src1, float *src2, float *dst);
INLINE float vdot(float *v1, float *v2);
INLINE void vcross(float *v1, float *v2, float *cross);
INLINE void vreflect(float *in, float *mirror, float *out);
INLINE void vtransform(float **m, float *v);
INLINE void vtransform4(float **m, float *v);
INLINE void calc_normal(float *norm, float *a, float *b, float *c);

/* Handy typedefs for declaring storage of 2- & 3-vectors. */

typedef float XY[2];
typedef float XYZ[3];

/* Routines for the quaternion representation of rotations. [quat.c] */

typedef struct {
	float s;
	float v[3];
} QUAT;

extern void axis_to_quaternion(float *axis, float theta, QUAT *quat);
extern void qmult(QUAT *q1, QUAT *q2, QUAT *dest);
extern void qnormalize(QUAT *q);
extern void quaternion_to_rotmatrix(QUAT *q, float *m);

#ifdef  __cplusplus
}
#endif

#define __include_xyz
#include "xyz.c"
#undef __include_xyz
#undef INLINE

#endif  /* _UTIL_H */
